﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;

namespace CountDownTimer
{
    public static class PowerControl
    {
        /// <summary>
        /// 今すぐ再起動します。
        /// </summary>
        public static void Restart()
        {
            Shutdown(true);
        }

        /// <summary>
        /// 今すぐ電源を切ります。
        /// </summary>
        public static void Shutdown()
        {
            Shutdown(false);
        }

        /// <summary>
        /// 今すぐ電源を切ります。引数に true を渡すと再起動になります。
        /// </summary>
        /// <param name="restart"></param>
        public static void Shutdown(bool restart)
        {
            SilentStart("shutdown", (restart ? "-r" : "-s") + " -t 0 -f");
        }

        /// <summary>
        /// ログオフします。
        /// </summary>
        public static void Logoff()
        {
            SilentStart("logoff", "Console");
        }

        /// <summary>
        /// スタンバイ状態にします。
        /// </summary>
        public static void Suspend()
        {
            SetSuspendState(false, false, false);
        }

        /// <summary>
        /// 休止状態、又はスタンバイ状態にします。
        /// </summary>
        public static void Hibernate()
        {
            SetSuspendState(true, false, false);
        }

        /// <summary>
        /// 何も表示せずにコマンドを実行します。
        /// </summary>
        /// <param name="command"></param>
        /// <param name="args"></param>
        private static void SilentStart(string command, string args)
        {
            ProcessStartInfo ps = new ProcessStartInfo(command, args);
            ps.CreateNoWindow = true;
            ps.UseShellExecute = false;

            Process.Start(ps);
        }

        [System.Runtime.InteropServices.DllImport("Powrprof.dll", SetLastError = true)]
        public static extern bool SetSuspendState(bool hibernate, bool forceCritical, bool disableWakeEvent);
    }
}
